# coding: utf8

from functools import partial

from .yaml_value import get_value, string_data
from .base import separate, till, multi, choice, right, empty, choice_one, clear_empty_line


def get_indent(string):
    if string and string[0] == " ":
        return 1 + get_indent(string[1:])
    return 0


def equal_indent(a, indent):
    return get_indent(a) == indent


def split_line(row):
    return row.startswith("---")


def remove_comment(string):
    return "\n".join([row for row in string.split("\n") if not split_line(row)])


def multi_string_data(string):
    def collector(stream):
        value, stream = separate(stream, "\n")
        return string_data(value) if value else "", stream

    filtered = clear_empty_line(string)
    result, tail = till(
        filtered,
        collector,
        lambda x: equal_indent(x, get_indent(filtered)) or x.lstrip(" ").startswith("\n"),
        False
    )
    if len(filtered) < len(string):
        result.insert(0, "")
    return "\n".join(result).rstrip(), tail


def pair_with_simple_data_value(string):
    def _get_pair(stream):
        head, tail = separate(stream, ": ")
        enum, tail = separate(tail, "\n")
        if enum.strip() in ("|", ">"):
            value, tail = multi_string_data(tail)
        else:
            value = get_value(enum)
        return (string_data(head), value), tail

    return choice(separate(string, "\n")[0].find(": ") != -1, _get_pair, empty)(string)


def pair_with_complicate_data_value(string):
    def _get_pair(stream):
        key, tail = separate(stream, ":\n")
        value, tail = choice(tail.strip().startswith("-"), get_list, get_dict)(tail)
        return (string_data(key), value), tail

    return choice(separate(string, "\n")[0].strip().endswith(":"), _get_pair, empty)(string)


def pair_data(string):
    return choice_one(string, pair_with_complicate_data_value, pair_with_simple_data_value)


def list_data(string):
    result = choice(string.lstrip().startswith("-"), get_list, empty)(string)
    return result[0] if result else None


def get_list(string):
    def collector(stream):
        result, stream = separate(right(*separate(stream, "-")), "\n")
        if result.strip() in ("|", ">"):
            return multi_string_data(stream)
        return get_value(result), stream

    string = clear_empty_line(string)
    return till(string, collector, partial(equal_indent, indent=get_indent(string)))


def get_dict(string):
    string = clear_empty_line(string)
    enum, tail = till(string, pair_data, partial(equal_indent, indent=get_indent(string)))
    return dict(enum), tail


def loads(string):
    string = remove_comment(string.rstrip())
    return choice_one(string, lambda x: list_data(x), lambda x: dict(multi(clear_empty_line(x), pair_data)))


def load(file_obj):
    return loads(file_obj.read())



# coding: utf8

import unittest

from toyyaml.yaml_value import get_enum, get_value


class TestYamlValue(unittest.TestCase):
    def test_get_simple_enum(self):
        enum, string = get_enum("aaa", ",")
        self.assertEquals(enum, "aaa")
        self.assertEquals(string, "")

    def test_get_enum_split_by_comma(self):
        enum, string = get_enum("aaa, aaa", ",")
        self.assertEquals(enum, "aaa")
        self.assertEquals(string, " aaa")

    def test_get_int_value(self):
        enum = get_value("  23 ")
        self.assertEquals(enum, 23)

    def test_get_float_value(self):
        enum = get_value("  23.0 ")
        self.assertEquals(enum, 23.0)


# coding: utf8

import unittest

from toyyaml.toyyaml import pair_data, loads, multi_string_data


class TestYaml(unittest.TestCase):
    def test_get_pair_with_simple_dict(self):
        pair, string = pair_data("aaa: bbb")
        self.assertEquals(pair, ("aaa", "bbb"))
        self.assertEquals(string, "")

    def test_success_split_key_and_value_when_key_and_value_has_quote(self):
        pair, string = pair_data("aa:a: bb:b")
        self.assertEquals(pair, ("aa:a", "bb:b"))
        self.assertEquals(string, "")

    def test_get_pair_and_popup_string(self):
        pair, string = pair_data("aaa: bbb\nasdasd")
        self.assertEquals(pair, ("aaa", "bbb"))
        self.assertEquals(string, "asdasd")

    def test_get_multi_line_string_pair_value(self):
        pair, string = pair_data("""aaa: |
        asdasd
        bbbbb
        """)
        self.assertEquals(pair, ("aaa", "asdasd\nbbbbb"))
        self.assertEquals(string, "")

    def test_get_pair_with_value_is_list(self):
        pair, string = pair_data("aaa: [bb, cc, dd]")

        self.assertEquals(pair, ("aaa", ["bb", "cc", "dd"]))
        self.assertEquals(string, "")

    def test_get_pair_with_value_is_dict(self):
        pair, string = pair_data("""aaa:
            bb: cc
            dd: ee
        """)

        self.assertEquals(pair, ("aaa", dict(bb="cc", dd="ee")))
        self.assertEquals(string, "        ")

    def test_get_pair_with_value_is_yaml_list(self):
        pair, string = pair_data("""aaa:
            - bbcc
            - ddee
        """)

        self.assertEquals(pair, ("aaa", ["bbcc", "ddee"]))
        self.assertEquals(string, "        ")

    def test_load_list_datas(self):
        result = loads("""
        - aaa

        - bbb
        """)
        self.assertEquals(len(result), 2)
        self.assertEquals(result, ["aaa", "bbb"])

    def test_success_get_multi_line_string_value_when_with_empty_line(self):
        result, tail = multi_string_data("""
            asd
            aaa

            bbb
        - t
        """)
        self.assertEquals(result, "\nasd\naaa\n\nbbb")
        self.assertEquals(tail, "        - t\n        ")

    def test_load_read_data(self):
        result = loads("""
        project_name: mothership-admin

        tasks:
          asd-supasdponv:
            runner: buiasld_image
            workspace: troasds//webaasd/
            dockerfile: xxx/vvv.asd
            sasduasdffix: aasd

          asd-asd-asd:
            runner: asd
            workspace: tros/src/main/webapp/
            asdbb: asdasd/asdasdaf/asdasd.afasdasd
            asd: asdas

          123:
            asd: 12324
            aaa: asd/asd.asd
            ppp: asd

          unittest:
            - aaa
            - bb

          asdasd:
            asd: asd
            dockerfile: 22/33.release
            asdga: 111
            tag: dev
        """)
        self.assertEquals(len(result), 2)
        self.assertEquals(len(result["tasks"]), 5)



# coding: utf8


def left(a, _):
    return a


def right(_, b):
    return b


def empty(_):
    return None


def clear_empty_line(string):
    if string.startswith("\n"):
        return clear_empty_line(string[1:])
    if string.lstrip(" ").startswith("\n"):
        return clear_empty_line(string.lstrip(" ")[1:])
    return string


def multi(string, cut, skip_empty_line=True):
    result = list()
    while string:
        enum, string = cut(string)
        result.append(enum)
        string = clear_empty_line(string) if skip_empty_line else string
    return result


def till(string, cut, condition, skip_empty_line=True):
    result = list()
    while string and condition(string):
        enum, string = cut(string)
        result.append(enum)
        string = clear_empty_line(string) if skip_empty_line else string
    return result, string


def choice(condition, a, b):
    return a if condition else b


def choice_one(string, *args):
    def _get_result(stream):
        result = args[0](stream)
        return result if result is not None else choice_one(stream, *args[1:])
    return choice(args, _get_result, empty)(string)


def surround(string, start, end):
    return string[0] == start and string[-1] == end and string[1:-1]


def separate(string, symbol, reverse=False, padding=True):
    result = [enum for enum in choice(reverse, string.rsplit, string.split)(symbol, 1)]
    return fill(result) if padding else result


def fill(enums):
    return enums[0], enums[1] if len(enums) > 1 else ""



# coding: utf8

from functools import partial

from .base import multi, surround, choice_one, separate, choice, empty


def get_enum(string, separate_symbol):
    enum, string = separate(string, separate_symbol)
    return get_value(enum), string


def get_list(string, separate_symbol=","):
    return multi(string, partial(get_enum, separate_symbol=separate_symbol))


def string_data(string):
    result = surround(string, '"', '"')
    return result if result else string.strip()


def list_data(string):
    result = surround(string.strip(), "[", "]")
    return choice(result, get_list, empty)(result)


def int_data(string):
    try:
        return int(string)
    except ValueError:
        return None


def float_data(string):
    try:
        return float(string)
    except ValueError:
        return None


def empty_data(string):
    return None if string.strip() else ""


def get_value(string):
    return choice_one(string, empty_data, list_data, int_data, float_data, string_data)



# coding: utf8

from setuptools import setup


setup(
    name="toyyaml",
    version="0.1",
    description="simple YAML format reader",
    author="yeyuexia",
    author_email="yyxworld@gmail.com",
    url="https://github.com/yeyuexia/toyyaml",
    packages=["toyyaml"]
)
